'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createNodes = exports.createNodeFromData = exports.createId = exports.constructChildIds = exports.constructParentId = exports.extractTypeName = undefined;

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _objectWithoutProperties2 = require('babel-runtime/helpers/objectWithoutProperties');

var _objectWithoutProperties3 = _interopRequireDefault(_objectWithoutProperties2);

var _values = require('babel-runtime/core-js/object/values');

var _values2 = _interopRequireDefault(_values);

var _crypto = require('crypto');

var _crypto2 = _interopRequireDefault(_crypto);

var _ramda = require('ramda');

var _constants = require('./constants');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Get the type name back from a formatted type name.
const extractTypeName = exports.extractTypeName = t => {
  /*
  t = singular(/(.+)(?:s|es)/gi.exec(t)[1]);
  if (t == "tagFamilie") {
    t = "tagFamilies";
  }
  */
  return t;
};

/**
 * Create parent id for elements.
 * 
 * @param {*} e Element data to be used
 * @param {*} type Type of the element
 */
const constructParentId = exports.constructParentId = (e, type) => {
  if (e.parent && e.parent.uuid) {
    return createId(e.parent.uuid, type);
  }
  return null;
};

/**
 * Create array with ids of children.
 * 
 * @param {*} e Element data to be used
 * @param {*} type Type of the element
 */
const constructChildIds = exports.constructChildIds = (e, type) => {
  if (e.children && e.children.elements) {
    return (0, _values2.default)(e.children.elements).map(function (v) {
      return createId(v.uuid, type);
    });
  } else {
    return [];
  }
};

/**
 * Constructs a global gatsby id from the element uuid and type.
 * 
 * @param {*} uuid 
 * @param {*} type 
 */
const createId = exports.createId = (uuid, type) => {
  return _crypto2.default.createHash(`md5`).update(uuid + ":" + type).digest(`hex`);
};

/**
 * Create a gatsby node from the input value.
 * 
 * @param {*} createNode Gatsby create method
 * @param {*} reporter Logger
 * @param {*} value Object which holds the element data
 * @param {*} key Key or type of the element
 */
const createNodeFromData = (createNode, reporter, value, key) => {
  // rename fields property since 'fields' is reserved by gatsby
  if (value.fields) {
    value.data = value.fields;
    delete value.fields;
  }
  if (!value.uuid) {
    reporter.panic(`The object for property ${key} did not contain a uuid property. This property is mandatory in order to setup the gatsby nodes. Omitting value.`);
    return;
  }
  value.id = createId(value.uuid, key);

  const id = value.id,
        fields = (0, _objectWithoutProperties3.default)(value, ['id']);

  const jsonNode = (0, _stringify2.default)(value);

  const gatsbyNode = (0, _extends3.default)({
    id: value.id
  }, fields, {
    parent: constructParentId(value, key),
    //children: constructChildIds(value, key),
    children: [],
    internal: {
      type: key,
      content: jsonNode,
      contentDigest: _crypto2.default.createHash(`md5`).update(jsonNode).digest(`hex`)
    }
  });

  if (_constants.DEBUG_MODE) {
    const jsonFields = (0, _stringify2.default)(fields, null, 2);
    const jsonGatsbyNode = (0, _stringify2.default)(gatsbyNode, null, 2);
    reporter.info(`  processing node: ${jsonNode}`);
    reporter.info(`    node id ${id}`);
    reporter.info(`    node fields: ${jsonFields}`);
    reporter.info(`    gatsby node: ${jsonGatsbyNode}`);
  }

  createNode(gatsbyNode);
  return gatsbyNode;
};

/**
 * Creates nodes from the input value. This method will also handle arrays of input data.
 * 
 * @param {*} boundActionCreators
 * @param {*} reporter Logger
 */
exports.createNodeFromData = createNodeFromData;
const createNodes = exports.createNodes = (boundActionCreators, reporter) => (value, key) => {
  const createNode = boundActionCreators.createNode,
        createParentChildLink = boundActionCreators.createParentChildLink;

  key = extractTypeName(key);

  // Check whether the found value contains multiple elements
  if (value.elements) {
    let relations = [];
    let nodes = {};
    (0, _ramda.forEach)(queryResultNode => {

      let gatsbyNode = createNodeFromData(createNode, reporter, queryResultNode, key);

      // Store relational information about nodes so that we can later constuct the links
      if (key == "nodes") {
        let childrenIds = constructChildIds(queryResultNode, key);
        relations.push({ "parentId": gatsbyNode.id, "childrenIds": childrenIds });
        nodes[gatsbyNode.id] = gatsbyNode;
      }
    }, value.elements);

    // Now that all nodes have been created we can setup the relations
    (0, _ramda.forEach)(rel => {
      (0, _ramda.forEach)(childId => {
        let child = nodes[childId];
        let parent = nodes[rel.parentId];
        if (child && parent) {
          createParentChildLink({ parent: parent, child: child });
        } else {
          reporter.warn("Unable to create parent-child relationship between " + rel.parent + " and " + childId);
        }
      }, rel.childrenIds);
    }, relations);
  } else {
    createNodeFromData(createNode, reporter, value, key);
  }
};